import "Turbine";
import "Turbine.Debug";
import "Turbine.Gameplay";
import "Turbine.UI";
import "Turbine.UI.Lotro";
import "DhorPlugins.Extensions";
import "DhorPlugins.Utils.BitOps";


--[[ This is the main window for the Travel Window UI	                      ]]--
--[[ It handles the loading and saving the settings,	                      ]]--
--[[ creating the lists of travel shortcuts, creates	                      ]]--
--[[ and handles the context menu.						                         ]]--
--[[ Functions:											                            ]]--
--[[		Constructor() - initializes the window and	                      ]]--
--[[			creates all the sub controls			                            ]]--
--[[		LoadSettings() - loads user settings		                         ]]--
--[[		SaveSettings() - saves the user's settings	                      ]]--
--[[		UpdateSettings() - updates the user's		                         ]]--
--[[			settings when the context menu changes	                         ]]--
--[[		CountRations() - checks the user's bags to	                      ]]--
--[[			see how many travel rations they have	                         ]]--
--[[		SetMapHome() - creates a window that the                           ]]--
--[[			player can use to set their map home	                         ]]--
--[[			shortcut								                                  ]]--
--[[		SaveMapHome() - updates the travel			                         ]]--
--[[			locations, and saves the player's map	                         ]]--
--[[			home shortcut data						                            ]]--

TravelWindow = class( Turbine.UI.Lotro.Window);

function TravelWindow:Constructor()
	Turbine.UI.Lotro.Window.Constructor( self );
	
	settings = { };
	travelShortcuts = { };         -- put all the shortcuts into one table
	self.minWidth = 220;
	self.minHeight = 150;
	self.disableResize = false;
	self.options = nil;
	
	-- create the lists of travel locations and the shortcuts
	-- that are used to execute them
	TravelInfo = TravelDictionaries();
	self:LoadSettings();
	
	-- if the player has set the map home skill, insert that
	-- into the list of locations
	if(settings.mapHome ~= nil) then
		genLocations:InsertData(1,"Map Home",settings.mapHome);
	end
	
	-- if the player has a PvMP map, then insert it into the list
	if(settings.mapGlanVraig ~= nil) then
	   genLocations:InsertData(2,"Gln Vraig Map",settings.mapGlanVraig);
	end
	
	-- redo the counts
	TravelInfo:GetCounts();   

   self:CheckEnabledSettings();   
   self:SetShortcuts();   

	-- configure the main window
	self:SetPosition(settings.positionX,settings.positionY);	
	self:SetSize( settings.width, settings.height );
	self:SetBackColor( Turbine.UI.Color(0.0,0,0,0) );
	self:SetText( "Travel" );
	self:SetOpacity( 1 );

	-- create a single context menu to use on all panels
	menu = SettingsMenu(self);
	menu:SetSettings(settings.race,
                     settings.class,
                     settings.mode,
                     settings.filters);
		
	-- create the Tabbed Panel to hold all the other panels
	self.MainPanel = DhorPlugins.Extensions.DPanel();
	self.MainPanel:SetOpacity(1);
	self.MainPanel:SetSize(settings.width - 20,settings.height - 60);
	self.MainPanel:SetPosition(10,40);
	self.MainPanel:SetButtonsVisible(false)  -- make sure to hide the tab buttons
	self.MainPanel:SetParent(self);
	--self.MainPanel:SetBackColor(Turbine.UI.Color(1,1,1,1));
	self.MainPanel:SetVisible(true);
	
	-- create the tabs for each mode of display
	self.ListTab = TravelListTab();	
	self.GridTab = TravelGridTab();
	self.CaroTab = TravelCaroTab();
   self.PullTab = TravelPulldownTab();	
	
	-- add the tabs to the panel
	self.MainPanel:AddTab(self.ListTab);
	self.MainPanel:AddTab(self.GridTab);
	self.MainPanel:AddTab(self.CaroTab);
	self.MainPanel:AddTab(self.PullTab);
                         
   -- create a control for resizing the window
   self.sizeControl = Turbine.UI.Control();
   self.sizeControl:SetParent(self);
   self.sizeControl:SetZOrder(100);
   self.sizeControl:SetSize(20,20);
   self.sizeControl:SetBackColorBlendMode(Turbine.UI.BlendMode.Screen)
   self.sizeControl:SetBackColor(Turbine.UI.Color(0.1,1,1,1));
   self.sizeControl:SetPosition(self:GetWidth() - self.sizeControl:GetWidth(),
                            self:GetHeight() - self.sizeControl:GetHeight()); 
                             
	-- track if the number of rations the player has changes
	self.rationCount = 0;
	self.oldRationCount = 1;
	
	-- display the tab that was last selected
	self.MainPanel:SetTab(settings.mode);
   self.MainPanel:SetSize(self:GetWidth() - 20,self:GetHeight() - 60);
   self.MainPanel:UpdateTabs();
   self:UpdateSettings();
   
   -- timers for events
	self.lastTime = 0;
   self.lastMove = 0;   
   
--[[ Event Handlers ]]--

	-- make sure we listen for key presses
	--self:SetWantsKeyEvents( true );
	self:SetWantsUpdates(true);
	
	-- check if our position has changed, and save the settings if so
	self.PositionChanged = function(sender, args )
		one,two = self:GetPosition();
		settings.positionX = one;
		settings.positionY = two;
		self:SaveSettings();
		self.PullTab:ClosePulldown();
	end
	
	-- check the ration count once every 5 seconds
	-- or imediately on startup, and only if the
	-- window is visible
	
	self.Update = function( sender,args)
		if(Turbine.Engine.GetGameTime() - self.lastTime > 5 
               and self:IsVisible() or self.lastTime == 0) then
			self:CountRations();	
			self.lastTime = Turbine.Engine.GetGameTime();
		end
      

	end
	
	self.VisibleChanged = function(sender, args)
      if(self:IsVisible() == false) then
		    self.PullTab:ClosePulldown();
		end
	end
	
	-- change the background color of the resize button on entry to help
	-- show exactly where the button is
	self.sizeControl.MouseEnter = function(sender,args)	      	  
     self.sizeControl:SetBackColor(Turbine.UI.Color(0.9,1,1,1));
	end
	
	-- return background color to normal on mouse leave
	self.sizeControl.MouseLeave = function(sender,args)
	   self.sizeControl:SetBackColor(Turbine.UI.Color(0.1,1,1,1));
	end
	
	-- check if someone is going to start dragging the resize control
	self.sizeControl.MouseDown = function(sender,args)
	
	  -- check if resize has been disabled
	  if(self.disableResize) then
	     return;
	  end  
	
	  -- set start location of resize and enable dragging
	  sender.dragStartX = args.X;
	  sender.dragStartY = args.Y;
	  sender.dragging = true;
	end
	
	self.sizeControl.MouseUp = function(sender,args)
	  -- disable dragging, then update settings
	  sender.dragging = false;	  
	  
	  settings.width = self:GetWidth();
	  settings.height = self:GetHeight();	  
	  self:UpdateSettings();
	end
	
	-- perform the resize
	self.sizeControl.MouseMove = function(sender,args)	
	  	 local width, height = self:GetSize();
      
         -- if resize dragging, then resize the window 		
		 if ( sender.dragging ) then
			   self:SetSize( width + ( args.X - sender.dragStartX ), 
                        height + ( args.Y - sender.dragStartY ) );
		 end
		 
		 -- update the size of the window
		 self:UpdateSize();
             
   	-- check time since last move
	   if(Turbine.Engine.GetGameTime() - self.lastMove > 0.1
               and self:IsVisible() or self.lastMove == 0) then   
      
         -- update the tabbed panels                   
         self.MainPanel:SetSize(self:GetWidth() - 20,self:GetHeight() - 60);
	      self.MainPanel:UpdateTabs();
         self.lastMove = Turbine.Engine.GetGameTime();
      end
      
	end  
end

function TravelWindow:UpdateSize()
         -- check that the window is not smaller than min width	
		 if (self:GetWidth() < self.minWidth) then
		       self:SetWidth(self.minWidth);
		 end
		
		 -- check that the window is not smaller than min height
		 if (self:GetHeight() < self.minHeight) then
		       self:SetHeight(self.minHeight);
       end
      
       -- set the new location of the resize control
		 self.sizeControl:SetPosition( self:GetWidth() - self.sizeControl:GetWidth(), 
                           self:GetHeight() - self.sizeControl:GetHeight() );
end  

function TravelWindow:LoadSettings()
	-- load the self.settings.  If a value is not available, set a default value
	settings = Turbine.PluginData.Load( Turbine.DataScope.Character, 
                                                "TravelWindowSettings" );

	if ( type( settings ) ~= "table" ) then
		settings = { };
	end
	
	if ( not settings.width) then
	   settings.width = self.minWidth;
	end
	
	if (not settings.height) then
	   settings.height = self.minHeight;
	end

	if ( not settings.positionX ) then
		settings.positionX = Turbine.UI.Display.GetWidth()
                                           - self:GetWidth() - 50; 
	end

	if ( not settings.positionY ) then
		settings.positionY = Turbine.UI.Display.GetHeight()
                                              - self:GetHeight() - 50 * 1.5;
	end

	if ( not settings.class ) then
		settings.class = 1;
	end

	if ( not settings.race ) then
		settings.race = 1;
	end

	if ( not settings.mode ) then
		settings.mode = 1;
	end

	if ( not settings.filters ) then
		settings.filters = 0x0F;
	end
   
	if ( not settings.enabled ) then	   
	  settings.enabled = { };
	end
	
	if ( not settings.order ) then
	   settings.order = { };
	end
	
	-- this is to set a specific order of the skills
	-- and is not working yet
	--[[
	travelLocationCount = TravelInfo:GetTotalCount();
	if(settings.mapHome ~= nil) then
	   travelLocationCount = travelLocationCount + 1;
   end
   if(settings.mapGlanVraig ~= nil) then
      travelLocationCount = travelLocationCount + 1;
	end
   	
	if ( not settings.order ) then
		settings.order = { };
		for i = 1, travelLocationCount, 1 do
			table.insert(settings.order,i);
		end
	else
	   Turbine.Shell.WriteLine(travelLocationCount);
	   Turbine.Shell.WriteLine(table.getn(settings.order));
		if (table.getn(settings.order) ~= travelLocationCount) then
			-- do something to append new data
			difference = travelLocationCount - table.getn(settings.order)
			for j = 1, difference, 1 do			
			   table.insert(settings.order,table.getn(settings.order) + 1);
			end
		end
	end
   ]]--
end

function TravelWindow:SaveSettings()
	-- save the settings
	Turbine.PluginData.Save( Turbine.DataScope.Character,
                                  "TravelWindowSettings", settings );
end

function TravelWindow:UpdateSettings()

	-- get the settings from the menu
	settings.race,
      settings.class,settings.mode,settings.filters = menu:GetSettings();

	-- save the settings
	self:SaveSettings();
	
	-- set which page of the tab panel to show	
	self.MainPanel:SetTab(settings.mode);
	
	-- update the page that is showing
	if(settings.mode == 1) then
	   self.minWidth = 245;
	   self.minHeight = 150;
		self.ListTab:SetItems();
	elseif(settings.mode == 2) then
	   self.minWidth = 120;
	   self.minHeight = 130;
		self.GridTab:SetItems();
	elseif(settings.mode == 3) then
	   self.minWidth = 120;
	   self.minHeight = 130;
	   self.CaroTab:SetItems();
	else
	   self.minWidth = 220;
	   self.minHeight = 150;
		self.PullTab:SetItems();
	end
   
   self:UpdateSize();
   
   self.MainPanel:SetSize(self:GetWidth() - 20,self:GetHeight() - 60);
	self.MainPanel:UpdateTabs(); 		
end


function TravelWindow:CountRations()

	-- get the player's inventory
	player = Turbine.Gameplay.LocalPlayer();
	backpack = player:GetBackpack();

	-- scan the inventory for specific image IDs.
	-- when the icon image and background image match
	-- then we are fairly confident we have found 
	-- the travel rations, add the quantity to the total
	self.rationCount = 0;
	for i = 1, backpack:GetSize(),1 do
		local item = backpack:GetItem(i);

		if(item ~= nil) then
			-- this will be where we check it its a travel ration
			if(string.find(item:GetName(),"Travelling Ration")) then
				self.rationCount = self.rationCount + item:GetQuantity();
			end
		end
	end

	-- if the ration count has changed since last
	-- checked, then update all the tabs with the
	-- new value
	if(self.rationCount ~= self.oldRationCount) then		
		self.ListTab:SetRationCount(self.rationCount);
		self.GridTab:SetRationCount(self.rationCount);
		self.CaroTab:SetRationCount(self.rationCount);
		self.PullTab:SetRationCount(self.rationCount);
		
		self.oldRationCount = self.rationCount;
	end
end

function TravelWindow:SetMapHome()

   -- disable the resize while the map update window is open
   -- also close the options window
   self.disableResize = true;
   self:CloseOptions();

	-- create the window used to add the map
	--self.MapWindow = Turbine.UI.Extensions.SimpleWindow();
	self.MapWindow = Turbine.UI.Control();
	self.MapWindow:SetPosition(10,35);
	self.MapWindow:SetSize(settings.width - 20,settings.height - 60);
	self.MapWindow:SetBackColor(Turbine.UI.Color(1,0.1,0.1,0.1));
	self.MapWindow:SetParent(self);
	self.MapWindow:SetZOrder(300);
	
	-- add an label to the window for instructions
	self.mapLabel = Turbine.UI.Label();
	self.mapLabel:SetForeColor(Turbine.UI.Color(1,0.2,0.2,0.6));
	self.mapLabel:SetPosition(0,15);
	self.mapLabel:SetSize(settings.width - 20,self.MapWindow:GetHeight() - 90);
	self.mapLabel:SetParent(self.MapWindow);
	self.mapLabel:SetVisible(true);
	self.mapLabel:SetFont(Turbine.UI.Lotro.Font.Verdana14);
	self.mapLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	self.mapLabel:SetMultiline(true);
	self.mapLabel:SetText("Drag your maps from your inventory onto the " ..
                           "quickslot below.  Then press the button " ..
	                        "to set your map home skills.");	
	
	-- add an empty quickslot to the window
	self.mapQuickSlot1 = Turbine.UI.Lotro.Quickslot();
	self.mapQuickSlot1:SetPosition(self.MapWindow:GetWidth() / 2 - 18,
                                          self.MapWindow:GetHeight() - 80);
	self.mapQuickSlot1:SetSize(36,36);
	self.mapQuickSlot1:SetParent(self.MapWindow);
	self.mapQuickSlot1:SetVisible(true);
	
	-- add a button to the window
	self.mapButton = Turbine.UI.Lotro.Button(); 
   self.mapButton:SetSize(60,50);
	self.mapButton:SetPosition(self.MapWindow:GetWidth() / 2 - 
                                          self.mapButton:GetWidth() / 2,
                                          self.MapWindow:GetHeight() - 35);
	self.mapButton:SetText("OK");
	self.mapButton:SetParent(self.MapWindow);
	self.mapButton:SetVisible(true);

	-- when the button is pressed, update the settings
	-- with the new skill, and reset everything
	self.mapButton.Click = function(sender,args)

		-- check if there is a shortcut in the quickslot
		if(self.mapQuickSlot1:GetShortcut():GetType() ~= 
                                 Turbine.UI.Lotro.ShortcutType.Undef) then
		
			-- save the shortcut data to the settings
			self:SaveMapHome(self.mapQuickSlot1:GetShortcut());
		end     		
		
		-- enable resize again
		self.disableResize = false;
		
		-- update the shortcuts list
		self:CheckEnabledSettings()
		self:SetShortcuts();
		self:UpdateSettings();
		
		-- close this window
		self.MapWindow:SetVisible(false);
		self.MapWindow = nil;
	end	
end

function TravelWindow:SaveMapHome(shortcut)

   local mapItem = shortcut:GetItem();
   if(mapItem:GetName() == "Map Home") then
   
     	-- remove the old shortcut if it exists
	  if(genLocations:IndexByKey("Map Home") == 1) then
		 genLocations:RemoveAtIndex(1);
	  end
	  
     	-- set the value
	  settings.mapHome = shortcut:GetData();
	  
  	   -- update the location lists	
	  genLocations:InsertData(1,"Map Home",settings.mapHome);
	  TravelInfo:GetCounts();	  
	  
   elseif(string.find(mapItem:GetName(),"Map to Gln Vraig")) then

        	-- remove the old shortcut if it exists
	  if(genLocations:IndexByKey("Gln Vraig Map") == 2) then
		 genLocations:RemoveAtIndex(2);
	  end
	  
     	-- set the value
	  settings.mapGlanVraig = shortcut:GetData();
	  
  	   -- update the location lists	
	  genLocations:InsertData(2,"Gln Vraig Map",settings.mapGlanVraig);
	  TravelInfo:GetCounts();
   else
      Turbine.Shell.WriteLine("No maps detected, no updates made."); 

	end	
	-- update and save everything	
	self:UpdateSettings();
end

-- for debug purposes
function TravelWindow:DoDump()
	Turbine.Debug.Table.Dump(DhorPlugins.Travel);
end

function TravelWindow:SetShortcuts()
   travelShortcuts = { };
   counter = 1;
   
 	-- set the generic travel items
	for i = 1,travelCount[3],1 do
	   -- get the order number for the item
	   counter = self:TableIndex(settings.order,genLocations:KeyAtIndex(i));
	
	   -- set the shortcut for the quickslot, check
	   -- if the shortcut is the map home or not
	   if(string.len(genLocations:DataAtIndex(i)) > 12) then		
		   table.insert(travelShortcuts,TravelShortcut(2.0,genLocations:DataAtIndex(i),genLocations:KeyAtIndex(i),1,counter,settings.enabled[genLocations:KeyAtIndex(i)]));
	   else		
		   table.insert(travelShortcuts,TravelShortcut(6.0,genLocations:DataAtIndex(i),genLocations:KeyAtIndex(i),1,counter,settings.enabled[genLocations:KeyAtIndex(i)]));
	   end
	end	
 		
	-- add the race travel to the list
	for i = 1,travelCount[5],1 do
	   counter = self:TableIndex(settings.order,racialLocations:KeyAtIndex(i));
	  table.insert(travelShortcuts,TravelShortcut(6.0,racialLocations:DataAtIndex(i),racialLocations:KeyAtIndex(i),2,counter,settings.enabled[racialLocations:KeyAtIndex(i)]));
   end	

	-- set the reputation travel items
	for i = 1,travelCount[4],1 do
	   counter = self:TableIndex(settings.order,repLocations:KeyAtIndex(i));
	   table.insert(travelShortcuts,TravelShortcut(6.0,repLocations:DataAtIndex(i),repLocations:KeyAtIndex(i),3,counter,settings.enabled[repLocations:KeyAtIndex(i)]));
	end 
	
	-- set the hunter travel items
	for i = 1,travelCount[1],1 do 
	   counter = self:TableIndex(settings.order,hunterLocations:KeyAtIndex(i));
	   table.insert(travelShortcuts,TravelShortcut(6.0,hunterLocations:DataAtIndex(i),hunterLocations:KeyAtIndex(i),4,counter,settings.enabled[hunterLocations:KeyAtIndex(i)]));
	end
	
	-- set the warden travel items
	for i = 1,travelCount[2],1 do 
	   counter = self:TableIndex(settings.order,wardenLocations:KeyAtIndex(i));
	   table.insert(travelShortcuts,TravelShortcut(6.0,wardenLocations:DataAtIndex(i),wardenLocations:KeyAtIndex(i),5,counter,settings.enabled[wardenLocations:KeyAtIndex(i)]));
	end
   
   self:SortShortcuts()  
end

function TravelWindow:CheckEnabledSettings()
   --settings.enabled = { };
   -- need to find the highest sort number now    
   local counter = table.getn(settings.order) + 1;
   
	for i = 1,travelCount[3],1 do
	   if(settings.enabled[genLocations:KeyAtIndex(i)] == nil) then
         settings.enabled[genLocations:KeyAtIndex(i)] = true;
      end
	   --if(settings.order[genLocations:KeyAtIndex(i)] == nil) then
	   if(self:TableContains(settings.order, genLocations:KeyAtIndex(i)) == false) then          
         table.insert(settings.order,counter,genLocations:KeyAtIndex(i));
         counter = counter + 1;
      end
   end
   for i = 1,travelCount[4],1 do
	   if(settings.enabled[repLocations:KeyAtIndex(i)] == nil) then
         settings.enabled[repLocations:KeyAtIndex(i)] = true;
      end
	   if(self:TableContains(settings.order, repLocations:KeyAtIndex(i)) == false) then
         table.insert(settings.order,counter,repLocations:KeyAtIndex(i));
         counter = counter + 1;
      end
   end
   for i = 1,travelCount[5],1 do
	   if(settings.enabled[racialLocations:KeyAtIndex(i)] == nil) then
         settings.enabled[racialLocations:KeyAtIndex(i)] = true;
      end 
	   if(self:TableContains(settings.order, racialLocations:KeyAtIndex(i)) == false) then
         table.insert(settings.order,counter,racialLocations:KeyAtIndex(i));
         counter = counter + 1;
      end
   end
   for i = 1,travelCount[1],1 do
	   if(settings.enabled[hunterLocations:KeyAtIndex(i)] == nil) then
         settings.enabled[hunterLocations:KeyAtIndex(i)] = true;
      end
	   if(self:TableContains(settings.order, hunterLocations:KeyAtIndex(i)) == false) then
         table.insert(settings.order,counter,hunterLocations:KeyAtIndex(i)); 
         counter = counter + 1;
      end
   end
   for i = 1,travelCount[2],1 do
	   if(settings.enabled[wardenLocations:KeyAtIndex(i)] == nil) then
         settings.enabled[wardenLocations:KeyAtIndex(i)] = true;
      end
	   if(self:TableContains(settings.order, wardenLocations:KeyAtIndex(i)) == false) then
         table.insert(settings.order,counter,wardenLocations:KeyAtIndex(i));
         counter = counter + 1;
      end
   end  
end

function TravelWindow:OpenOptions()
    self.options = DhorPlugins.Travel.OptionsWindow(self);
end

function TravelWindow:CloseOptions()
   if(self.options ~= nil) then
      self.options:SetVisible(false);
   end
   self.options = nil;
end

function TravelWindow:TableContains(data, element)
   for i , value in pairs(data) do
      if (value == element) then
         return true;
      end
   end
   return false;
end

function TravelWindow:TableIndex(data, element)
   for i , value in pairs(data) do
      if (value == element) then
         return i;
      end
   end
   return 0;
end

function TravelWindow:SortShortcuts()
if #travelShortcuts < 2 then
		print("Table does not have enough data in it")
		return
	end

	for i = 1, #travelShortcuts do --> array start to end
		for j = 2, #travelShortcuts do
			if travelShortcuts[j]:GetIndex() < travelShortcuts[j-1]:GetIndex() then -->switch
				temp = travelShortcuts[j-1]
				travelShortcuts[j-1] = travelShortcuts[j]
				travelShortcuts[j] = temp
			end
		end
	end

	return
end
